/***************************************************
 Gundam rx-78-2 MG 1/100 
 Created 21-01-2024
 By [Marco Mortari][marco.mortari@quantix.it]
 
 ****************************************************
 DFPlayer - A Mini MP3 Player For Arduino
 <https://www.dfrobot.com/product-1121.html>
 
 ***************************************************
/

/***********Notice and Trouble shooting***************
 1.Connection and Diagram can be found here https://content.instructables.com/F9R/1GEJ/LRW30CK6/F9R1GEJLRW30CK6.png?auto=webp&frame=1&width=1024&height=1024&fit=bounds&md=a18869c411a6056bdb991bb106e3884a

 2.This code is tested on Arduino Uno
 3. la funzione gestioneLed per la Testa non fa quello che dovrebbe in Pwm , si accende solo a 255 ho messo resistenza da 1k per diminuire la luminosità 
 ma via software anche con mills non va
 ****************************************************/

//Inclusione delle librerie
#include <L298N.h>
#include <Bounce2.h>
#include <Servo.h> 
#include "Arduino.h"
#include "DFRobotDFPlayerMini.h"
#include <SoftwareSerial.h>


const int pinLedMitra = A2; // Pin del LED per simulare l'effetto mitragliatrice
const int interval = 100;   // Intervallo di tempo tra le transizioni (in millisecondi)
unsigned long previousMillisMitra = 0; // Memorizza il tempo dell'ultima transizione per l'effetto mitragliatrice
unsigned long startMillis = 0;
const long duration = 4000;
const long blinkInterval = 100;

//const int ledEnable = 13; //il led on board ci mostrerà lo stato di attivazione dei motori non piu usato
const int pinSwEnable = 2;  //il bottone presente nel modulo joystick per sparare
const int pinServo = 12;  //  pin per il servo
const int pinLed = 9; // led testa
int ritardo = 150; // ritardo accensione led testa
unsigned long debounceDelay = 10; //millisecondi per il debonuce del bottone

// Motori dc
const int jX = A0;  //pin analogico che legge i valori per le X
const int EN_X = 3;
const int IN1_X = 4;
const int IN2_X = 5;
long speedX, valX, mapX;  //variabili di gestione movimenti motore X
unsigned int direzioneX;

const int jY = A1;  //pin analogico che legge i valori per le Y
const int EN_Y = 6;
const int IN1_Y = 7;
const int IN2_Y = 8;
long speedY, valY, mapY;  //variabili di gestione movimenti motore Y
unsigned int direzioneY;

const int maxSpeed = 25; //massima velocità settabile per il motore da settare al minimo per evitare rotazione braccia troppo veloce dipende dai motori
const int minSpeed = 0; //minima velocità settabile per il motore

const int treshold = 0;  //la lettura dei potenziometri non è mai affidabile al 100%, questo valore aiuta a determinare il punto da considerare come "Stai fermo" nei movimenti
long tresholdUp, tresholdDown;  //variabili di servizio per espletare il compito descritto sopra

//Servo per Testa
const int servoMinAngle = 60; // Limite minimo dell'angolo del servo fermo è a 90 e si puo spostare solo di 30 gradi a sx e dx
const int servoMaxAngle = 120; // Limite massimo dell'angolo del servo fermo è a 90 e si puo spostare solo di 30 gradi a sx e dx

boolean abilitato, muoviX, muoviY;  //variabili di gestione dei movimenti
boolean enable = false;

const uint8_t AVANTI = L298N::FORWARD;
const uint8_t DIETRO = L298N::BACKWARD;

Bounce btnEnable = Bounce();  //istanzia un bottone dalla libreria Bounce

//istanzia i motori
L298N motoreX(EN_X, IN1_X, IN2_X);
L298N motoreY(EN_Y, IN1_Y, IN2_Y);

Servo myServo;  // Oggetto per il controllo del servo

int ledBrightness = 0;  // Luminosità iniziale del LED Testa

DFRobotDFPlayerMini myDFPlayer;  //Df Player 
  SoftwareSerial softSerial(10, 11);  // RX, TX DF Player 


void setup() {

  //inizializza valori motori dc
  speedX = speedY = 0;
  //enable = false;

  // Inizializzazione del servo
  myServo.attach(pinServo);
  myServo.write(90);  // Posizione iniziale del servo

  //definizione delle modalità dei pin
  pinMode(ledEnable, OUTPUT);
  pinMode(pinSwEnable, INPUT_PULLUP); //l'input dello switch ha bisogno di essere settato come INPUT_PULLUP

  digitalWrite(ledEnable, enable); 

  //configura il bottone del joystick utilizzando la libreria Bounce
  btnEnable.attach(pinSwEnable);
  btnEnable.interval(debounceDelay);

  //calcola range valori entro i quali considerare la posizione del joystick come "Stai fermo"
  tresholdDown = (maxSpeed / 2) - treshold;
  tresholdUp = (maxSpeed / 2) + treshold;

  //configura parametri dei motori
  motoreX.setSpeed(minSpeed);
  motoreY.setSpeed(minSpeed);
 // inizializza il led testa  
  pinMode(pinLed, OUTPUT);
  analogWrite(pinLed, ledBrightness);
// inizializza il led mitra    
pinMode(pinLedMitra, OUTPUT);
  startMillis = millis();
gestioneLed(); // richiama la funzione

  // Configurazione DFPlayer Mini
  softSerial.begin(9600);
  Serial.begin(9600);
  if (!myDFPlayer.begin(softSerial, true, true)) {
    Serial.println(F("Unable to begin DFPlayer:"));
    Serial.println(F("1. Please recheck the connection!"));
    Serial.println(F("2. Please insert the SD card!"));
    while (true) {
      delay(0);
    }
  }
  Serial.println(F("DFPlayer Mini online."));
  myDFPlayer.volume(10);
  myDFPlayer.play(1);
}

void loop() {

  //esegui funzione di controllo e lettura del bottone che determina lo stato di abilitazione
  checkEnable();

  digitalWrite(ledEnable, enable);  //mostra stato di abilitazione tramite il led su pin 13

  //esegui lettura analogica dei valori provenienti dai potenziometri del joystick
  valX = analogRead(jX);
  valY = analogRead(jY);

  //mappa i valori letti in funzione della velocità minima e massima
  mapX = map(valX, 0, 1023, minSpeed, maxSpeed);
  mapY = map(valY, 0, 1023, minSpeed, maxSpeed);

  //esegui funzione di comando dei motori
  pilotaMotori(mapX, mapY);
  // esegue funzione del servo
  controlloServo();
  //Esegue suono mitragliatore quando si preme il joystick
  if (btnEnable.fell()) {
    // Quando il pulsante viene premuto
    // Riproduci un suono utilizzando il DFPlayer Mini
    myDFPlayer.play(4);  // suono mitratgliatore
    delay(1000);  // Aggiungi un ritardo per evitare ripetizioni rapide
  }

 // Effetto mitragliatrice sul LED su A2
  // if (enable) {
    
  //   effettoMitragliatrice();
  // }
effettoMitragliatrice();

  if (myDFPlayer.available()) {
    printDetail(myDFPlayer.readType(), myDFPlayer.read());
  }
}

void pilotaMotori(long mapX, long mapY) { //  funzione pilota motori

  if (mapX <= tresholdDown) {
    //x va indietro
    speedX = map(mapX, tresholdDown, minSpeed,   minSpeed, maxSpeed);
    muoviX = true;
    direzioneX = DIETRO;
  } else if (mapX >= tresholdUp) {
    //x va avanti
    speedX = map(mapX,  maxSpeed, tresholdUp,  maxSpeed, minSpeed);
    muoviX = true;
    direzioneX = AVANTI;
  } else {
    //x sta fermo
    speedX = 0;
    muoviX = false;
  }

  if (mapY <= tresholdDown) {
    //y va giù
    speedY = map(mapY, tresholdDown, minSpeed,   minSpeed, maxSpeed);
    muoviY = true;
    direzioneY = DIETRO;
  } else if (mapY >= tresholdUp) {
    //y va su
    speedY = map(mapY,  maxSpeed, tresholdUp,  maxSpeed, minSpeed);
    muoviY = true;
    direzioneY = AVANTI;
  } else {
    //y sta fermo
    speedY = 0;
    muoviY = false;
  }

  //if (enable) {
    if (muoviX) {
      motoreX.setSpeed(speedX);
      motoreX.run(direzioneX);
    } else {
      motoreX.stop();
    }

    if (muoviY) {
      motoreY.setSpeed(speedY);
      motoreY.run(direzioneY);
    } else {
      motoreY.stop();
    }
  //} 
  //else {
 //   motoreX.stop();
//    motoreY.stop();
//  }
}

void checkEnable() {// controlla se il tasto è stato premuto

  btnEnable.update();

  if (btnEnable.fell()) {
    enable = !enable;
  }

}
void controlloServo() {// controlla il servo
 // if (enable) {
    int servoAngle = map(valX, 0, 1023, servoMinAngle, servoMaxAngle);
    
    // Assicura che l'angolo del servo sia compreso nei limiti
    if (muoviX) {
      myServo.write(servoAngle);
    }
 // }
  // Se il joystick è disabilitato, il servo rimane nella posizione attuale
}
void gestioneLed() {// Led Testa ( non funziona bene è stata messa una resistenza da 1k per ridurre la luce)
 // if (enable && btnEnable.fell()) 
 {
    // Imposta gradualmente la luminosità da 0 a 50
    for (int i = 0; i <= 50; i++) {
      analogWrite(pinLed, i * 5); // Moltiplica per 5 per ottenere una scala da 0 a 255
      delay(ritardo);
    }
    
    // Mantieni il LED acceso a una luminosità bassa
    analogWrite(pinLed, 255); // Modifica questo valore secondo le tue preferenze
  }
 }
void effettoMitragliatrice() {
    unsigned long currentMillis = millis();

  // Controlla se il pulsante è premuto per avviare la sequenza
  if (digitalRead(pinSwEnable) == LOW) {
    startMillis = currentMillis;  // Resetta il tempo di inizio
  }

  Serial.print("startMillis: ");
  Serial.println(startMillis);
  Serial.print("currentMillis: ");
  Serial.println(currentMillis);

  // Controlla se è passato il tempo totale di lampeggio
  if (currentMillis - startMillis < duration) {
    // Lampeggia il LED ogni 100 millisecondi (0.1 secondo)
    if ((currentMillis - startMillis) % (2 * blinkInterval) < blinkInterval) {
      digitalWrite(pinLedMitra, HIGH);
    } else {
      digitalWrite(pinLedMitra, LOW);
    }
  } else {
    digitalWrite(pinLedMitra, LOW);
  }
}

 



void printDetail(uint8_t type, int value){
  switch (type) {
    case TimeOut:
      Serial.println(F("Time Out!"));
      break;
    case WrongStack:
      Serial.println(F("Stack Wrong!"));
      break;
    case DFPlayerCardInserted:
      Serial.println(F("Card Inserted!"));
      break;
    case DFPlayerCardRemoved:
      Serial.println(F("Card Removed!"));
      break;
    case DFPlayerCardOnline:
      Serial.println(F("Card Online!"));
      break;
    case DFPlayerUSBInserted:
      Serial.println("USB Inserted!");
      break;
    case DFPlayerUSBRemoved:
      Serial.println("USB Removed!");
      break;
    case DFPlayerPlayFinished:
      Serial.print(F("Number:"));
      Serial.print(value);
      Serial.println(F(" Play Finished!"));
      break;
    case DFPlayerError:
      Serial.print(F("DFPlayerError:"));
      switch (value) {
        case Busy:
          Serial.println(F("Card not found"));
          break;
        case Sleeping:
          Serial.println(F("Sleeping"));
          break;
        case SerialWrongStack:
          Serial.println(F("Get Wrong Stack"));
          break;
        case CheckSumNotMatch:
          Serial.println(F("Check Sum Not Match"));
          break;
        case FileIndexOut:
          Serial.println(F("File Index Out of Bound"));
          break;
        case FileMismatch:
          Serial.println(F("Cannot Find File"));
          break;
        case Advertise:
          Serial.println(F("In Advertise"));
          break;
        default:
          break;
      }
      break;
    default:
      break;
  }
  
}
